//
//  GraphLineView.m
//  Graph
//
//  Created by Lucas Torquato on 15/07/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

//  (0,0)------------> X (Data)
//   |
//   |
//   |
//   |
//   * Y (Value)

#import "GraphLineView.h"

#define PAD_WIDTH 30.0f
#define PAD_HEIGHT 30.0f

#define LABEL_HEIGHT 28.5f

@implementation GraphLineView

@synthesize points, ratioWidth, ratioHeight;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
    }
    return self;
}

- (void)drawRect:(CGRect)rect 
{
    [self setBackgroundShadow];
    [self createMockPoints];
    
    [self calcFrameGraphWithValues:points];
    
    [self createGraphLineWithArrayValues:points];
    [self addPointsInGraph:points];
    
    [self addDatasLabels];
    [self addValuesLabels];
    
    [self addHorizontalAxis];
    [self addVerticalAxis];
}

#pragma mark - Config View

- (void)setBackgroundShadow
{
    self.layer.masksToBounds = NO;
    self.layer.cornerRadius = 8;
    self.layer.shadowOffset = CGSizeMake(5, 5);
    self.layer.shadowRadius = 5;
    self.layer.shadowOpacity = 0.5;
}

#pragma mark - Mock Test Data

- (void)createMockPoints
{
    points = [[NSMutableArray alloc] init];
    
    NSValue *point1 = [NSValue valueWithCGPoint:CGPointMake(1.0, 1.0)];
    NSValue *point2 = [NSValue valueWithCGPoint:CGPointMake(2.0, 2.0)];
    NSValue *point3 = [NSValue valueWithCGPoint:CGPointMake(3.0, 3.0)];
    NSValue *point4 = [NSValue valueWithCGPoint:CGPointMake(4.0, 4.0)];
    NSValue *point5 = [NSValue valueWithCGPoint:CGPointMake(5.0, 5.0)];
    NSValue *point6 = [NSValue valueWithCGPoint:CGPointMake(6.0, 5.0)];
    NSValue *point7 = [NSValue valueWithCGPoint:CGPointMake(7.0, 5.0)];
    NSValue *point8 = [NSValue valueWithCGPoint:CGPointMake(8.0, 4.0)];
    NSValue *point9 = [NSValue valueWithCGPoint:CGPointMake(9.0, 3.0)];
    NSValue *point10 = [NSValue valueWithCGPoint:CGPointMake(10.0,2.0)];
    
    [points addObject:point1];
    [points addObject:point2];
    [points addObject:point3];
    [points addObject:point4];
    [points addObject:point5];
    [points addObject:point6];
    [points addObject:point7];
    [points addObject:point8];
    [points addObject:point9];
    [points addObject:point10];
    
}

#pragma mark - Frame Calc

- (void)calcFrameGraphWithValues:(NSMutableArray*)values
{
    CGFloat heightGraph = 0.0;
    CGFloat widthGraph = 0.0;
    for (NSValue *point in values) {
        CGPoint pointCG = [point CGPointValue];
        
        //set Width of Graph
        if (widthGraph < pointCG.x) {
            widthGraph = pointCG.x;
        } 
        
        //set Height of Graph
        if (heightGraph < pointCG.y) {
            heightGraph = pointCG.y;
        }
    }
    
    ratioHeight = (self.frame.size.height - (2*PAD_HEIGHT)) / heightGraph;
    ratioWidth = (self.frame.size.width - (2*PAD_WIDTH))/ points.count;
    
    //NSLog(@"RATIO H: %f",ratioHeight);
    //NSLog(@"RATIO W: %f",ratioWidth);
}

#pragma mark - Graph 

-(void)createGraphLineWithArrayValues:(NSMutableArray*)values
{
    CGContextRef c = UIGraphicsGetCurrentContext();
    CGFloat red[4] = {1.0f, 0.0f, 0.0f, 1.0f};
    CGContextSetStrokeColor(c, red);
    CGContextBeginPath(c);
    CGContextMoveToPoint(c, PAD_WIDTH , self.frame.size.height - PAD_HEIGHT);
    
    for (int i = 0; i < points.count; i++) {
        CGPoint pointCG = [[points objectAtIndex:i]CGPointValue];
                                  
        CGContextAddLineToPoint(c, (ratioWidth * (i + 1)) + PAD_WIDTH, self.frame.size.height - PAD_HEIGHT - (ratioHeight * pointCG.y));
        
        //NSLog(@"%d:%f",i,(ratioWidth * (i + 1)) + PAD_WIDTH);
        //NSLog(@"%d: %f",i, self.frame.size.height - PAD_HEIGHT - (ratioHeight * pointCG.y));
    }
    CGContextStrokePath(c);    
}


- (void)addPointsInGraph:(NSMutableArray*)values
{
    CGContextRef c = UIGraphicsGetCurrentContext();
    CGFloat red[4] = {1.0f, 0.0f, 0.0f, 1.0f};
    CGContextSetStrokeColor(c, red);
    CGContextBeginPath(c);
    CGContextMoveToPoint(c, PAD_WIDTH , self.frame.size.height - PAD_HEIGHT);
    
    for (int i = 0; i < points.count; i++) {
        CGPoint pointCG = [[points objectAtIndex:i]CGPointValue];
        
        CGContextFillRect(c,CGRectMake((ratioWidth * (i + 1)) + PAD_WIDTH - 2.5, self.frame.size.height - PAD_HEIGHT - (ratioHeight * pointCG.y) - 2.5, 5, 5));
        
        //NSLog(@"%d:%f",i,(ratioWidth * (i + 1)));
        //NSLog(@"%d: %f",i, self.frame.size.height - ((ratioHeight * pointCG.y) - 20));
    }
    CGContextStrokePath(c);    

}

#pragma mark - Labels

- (void)addDatasLabels
{
    for (int i = 0; i < points.count; i++) {
        UILabel *dataLabel = [[UILabel alloc] initWithFrame:CGRectMake((ratioWidth * (i + 1)) + PAD_WIDTH - 2.5, self.frame.size.height - PAD_HEIGHT, 24, 24)];
        
        CGPoint pointCG = [[points objectAtIndex:i]CGPointValue];

        [dataLabel setTextColor:[UIColor blueColor]];
        [dataLabel setBackgroundColor:[UIColor clearColor]];
        [dataLabel setFont:[UIFont fontWithName:@"Helvetica-Light" size:(10.0)]];
        [self addSubview:dataLabel];
        dataLabel.text = [NSString stringWithFormat: @"%0.f", pointCG.x];
    }
}

- (void)addValuesLabels
{
    CGFloat numberOfLabels = floorf((self.frame.size.height-(2*PAD_HEIGHT))/LABEL_HEIGHT);
    if (numberOfLabels) {
        
    }
    
    for (int i = 0; i < 8; i++) {
        UILabel *valueLabel = [[UILabel alloc] initWithFrame:CGRectMake(1, self.frame.size.height - (2*PAD_HEIGHT - 10) - (i*LABEL_HEIGHT) , 30, 30)];
        NSLog(@"VALUES - %f",valueLabel.frame.origin.y);
        
        [valueLabel setTextColor:[UIColor blueColor]];
        [valueLabel setBackgroundColor:[UIColor clearColor]];
        [valueLabel setFont:[UIFont fontWithName:@"Helvetica-Light" size:(10.0)]];
        [self addSubview:valueLabel];
        valueLabel.text = [NSString stringWithFormat: @"%d",i];
        NSLog(@"VALUES 2 - %f",valueLabel.frame.origin.y);
    }

}

#pragma mark - Eixo

- (void)addHorizontalAxis
{
    CGContextRef c = UIGraphicsGetCurrentContext();
    CGFloat red[4] = {0.0f, 0.0f, 0.0f, 1.0f};
    CGContextSetStrokeColor(c, red);
    CGContextBeginPath(c);
    CGContextMoveToPoint(c, PAD_WIDTH , self.frame.size.height - PAD_HEIGHT);
    
    CGContextAddLineToPoint(c, self.frame.size.width - PAD_HEIGHT, self.frame.size.height - PAD_WIDTH);

    CGContextStrokePath(c);  

}

- (void)addVerticalAxis
{
    CGContextRef c = UIGraphicsGetCurrentContext();
    CGFloat red[4] = {0.0f, 0.0f, 0.0f, 1.0f};
    CGContextSetStrokeColor(c, red);
    CGContextBeginPath(c);
    CGContextMoveToPoint(c, PAD_HEIGHT , PAD_HEIGHT);
    
    CGContextAddLineToPoint(c, PAD_HEIGHT, self.frame.size.height - PAD_HEIGHT);
    
    CGContextStrokePath(c);  
    
}

@end
